# Round v to x decimal places
def round_to(v)
  x = Sketchup.active_model.options["UnitsOptions"]["LengthPrecision"].to_i
  (v * 10**x).round.to_f / 10**x
end

def draw_cline(*pts)
  Sketchup.active_model.entities.add_cline(pts[0], pts[1])
end

def sideview
  model = Sketchup.active_model
  entities = model.entities
  # Set up 3 Arrays to store the vertices of each 
  # of the 3 views: top, front, and side.

  top_verts = []
  front_verts = []
  side_verts = []

  entities.each do |e|
    next unless e.typename == "Face"
    if e.normal == Z_AXIS or e.normal == Z_AXIS.reverse
      top_verts.concat e.vertices
      next
    end
    if e.normal == Y_AXIS.reverse or e.normal == Y_AXIS
      front_verts.concat e.vertices
      next
    end
    if e.normal == X_AXIS or e.normal == X_AXIS.reverse
      side_verts.concat e.vertices
      next
    end
  end

  lines = []

  top_verts.uniq.each   { |vert| lines.push [vert.position, Z_AXIS] }
  front_verts.uniq.each { |vert| lines.push [vert.position, Y_AXIS.reverse] }
  side_verts.uniq.each  { |vert| lines.push [vert.position, X_AXIS] }

  line_hash = Hash.new(0)

  l = lines.length
  for i in 0..l-2
    for j in (i+1)..l-1
      pt = Geom::intersect_line_line(lines[i], lines[j])
      next if pt.nil?
      draw_cline(lines[i][0], pt)
      draw_cline(lines[j][0], pt)
      pt = pt.to_a.map{|e| round_to(e)}
      line_hash[pt] += 1
    end
  end

  line_hash.each_pair do |pt, intersections|
    #puts "#{k.inspect}=#{v.inspect}"
    if intersections >= 3
      Sketchup.active_model.entities.add_cpoint(pt)
    end
  end

end #def sideview

UI.menu.add_item('sideview') { sideview }
